@@ -1,14 +1,13 @@
 # Taken from https://github.com/pytorch/audio/blob/master/torchaudio/models/wav2letter.py
 # So that we don't need torchaudio to be installed
 
-import torch
-from torch import Tensor
-from torch import nn
-import torch.nn.functional as F
-
 import math
 from collections import OrderedDict
-from typing import Tuple, Optional
+from typing import Optional, Tuple
+
+import torch
+import torch.nn.functional as F
+from torch import nn, Tensor
 
 __all__ = ["Wav2Letter"]
 
@@ -24,41 +23,77 @@ class Wav2Letter(nn.Module):
         num_features (int, optional): Number of input features that the network will receive (Default: ``1``).
     """
 
-    def __init__(self, num_classes: int = 40,
-                 input_type: str = "waveform",
-                 num_features: int = 1) -> None:
+    def __init__(
+        self, num_classes: int = 40, input_type: str = "waveform", num_features: int = 1
+    ) -> None:
         super().__init__()
 
         acoustic_num_features = 250 if input_type == "waveform" else num_features
         acoustic_model = nn.Sequential(
-            nn.Conv1d(in_channels=acoustic_num_features, out_channels=250, kernel_size=48, stride=2, padding=23),
+            nn.Conv1d(
+                in_channels=acoustic_num_features,
+                out_channels=250,
+                kernel_size=48,
+                stride=2,
+                padding=23,
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3),
+            nn.Conv1d(
+                in_channels=250, out_channels=250, kernel_size=7, stride=1, padding=3
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=250, out_channels=2000, kernel_size=32, stride=1, padding=16),
+            nn.Conv1d(
+                in_channels=250, out_channels=2000, kernel_size=32, stride=1, padding=16
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=2000, out_channels=2000, kernel_size=1, stride=1, padding=0),
+            nn.Conv1d(
+                in_channels=2000, out_channels=2000, kernel_size=1, stride=1, padding=0
+            ),
+            nn.ReLU(inplace=True),
+            nn.Conv1d(
+                in_channels=2000,
+                out_channels=num_classes,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+            ),
             nn.ReLU(inplace=True),
-            nn.Conv1d(in_channels=2000, out_channels=num_classes, kernel_size=1, stride=1, padding=0),
-            nn.ReLU(inplace=True)
         )
 
         if input_type == "waveform":
             waveform_model = nn.Sequential(
-                nn.Conv1d(in_channels=num_features, out_channels=250, kernel_size=250, stride=160, padding=45),
-                nn.ReLU(inplace=True)
+                nn.Conv1d(
+                    in_channels=num_features,
+                    out_channels=250,
+                    kernel_size=250,
+                    stride=160,
+                    padding=45,
+                ),
+                nn.ReLU(inplace=True),
             )
             self.acoustic_model = nn.Sequential(waveform_model, acoustic_model)
 
@@ -77,6 +112,7 @@ class Wav2Letter(nn.Module):
         x = nn.functional.log_softmax(x, dim=1)
         return x
 
+
 # Taken from  https://github.com/SeanNaren/deepspeech.pytorch with modifications
 class SequenceWise(nn.Module):
     def __init__(self, module):
@@ -96,9 +132,9 @@ class SequenceWise(nn.Module):
         return x
 
     def __repr__(self):
-        tmpstr = self.__class__.__name__ + ' (\n'
+        tmpstr = self.__class__.__name__ + " (\n"
         tmpstr += self.module.__repr__()
-        tmpstr += ')'
+        tmpstr += ")"
         return tmpstr
 
 
@@ -141,14 +177,27 @@ class InferenceBatchSoftmax(nn.Module):
 
 
 class BatchRNN(nn.Module):
-    def __init__(self, input_size, hidden_size, rnn_type=nn.LSTM, bidirectional=False, batch_norm=True):
+    def __init__(
+        self,
+        input_size,
+        hidden_size,
+        rnn_type=nn.LSTM,
+        bidirectional=False,
+        batch_norm=True,
+    ):
         super().__init__()
         self.input_size = input_size
         self.hidden_size = hidden_size
         self.bidirectional = bidirectional
-        self.batch_norm = SequenceWise(nn.BatchNorm1d(input_size)) if batch_norm else None
-        self.rnn = rnn_type(input_size=input_size, hidden_size=hidden_size,
-                            bidirectional=bidirectional, bias=True)
+        self.batch_norm = (
+            SequenceWise(nn.BatchNorm1d(input_size)) if batch_norm else None
+        )
+        self.rnn = rnn_type(
+            input_size=input_size,
+            hidden_size=hidden_size,
+            bidirectional=bidirectional,
+            bias=True,
+        )
         self.num_directions = 2 if bidirectional else 1
 
     def flatten_parameters(self):
@@ -161,7 +210,11 @@ class BatchRNN(nn.Module):
         x, h = self.rnn(x)
         x, _ = nn.utils.rnn.pad_packed_sequence(x)
         if self.bidirectional:
-            x = x.view(x.size(0), x.size(1), 2, -1).sum(2).view(x.size(0), x.size(1), -1)  # (TxNxH*2) -> (TxNxH) by sum
+            x = (
+                x.view(x.size(0), x.size(1), 2, -1)
+                .sum(2)
+                .view(x.size(0), x.size(1), -1)
+            )  # (TxNxH*2) -> (TxNxH) by sum
         return x
 
 
@@ -175,8 +228,15 @@ class Lookahead(nn.Module):
         self.context = context
         self.n_features = n_features
         self.pad = (0, self.context - 1)
-        self.conv = nn.Conv1d(self.n_features, self.n_features, kernel_size=self.context, stride=1,
-                              groups=self.n_features, padding=0, bias=None)
+        self.conv = nn.Conv1d(
+            self.n_features,
+            self.n_features,
+            kernel_size=self.context,
+            stride=1,
+            groups=self.n_features,
+            padding=0,
+            bias=None,
+        )
 
     def forward(self, x):
         x = x.transpose(0, 1).transpose(1, 2)
@@ -186,13 +246,28 @@ class Lookahead(nn.Module):
         return x
 
     def __repr__(self):
-        return self.__class__.__name__ + '(' \
-            + 'n_features=' + str(self.n_features) \
-            + ', context=' + str(self.context) + ')'
+        return (
+            self.__class__.__name__
+            + "("
+            + "n_features="
+            + str(self.n_features)
+            + ", context="
+            + str(self.context)
+            + ")"
+        )
+
 
 class DeepSpeech(nn.Module):
-    def __init__(self, rnn_type, labels, rnn_hidden_size, nb_layers, audio_conf,
-                 bidirectional, context=20):
+    def __init__(
+        self,
+        rnn_type,
+        labels,
+        rnn_hidden_size,
+        nb_layers,
+        audio_conf,
+        bidirectional,
+        context=20,
+    ):
         super().__init__()
 
         self.hidden_size = rnn_hidden_size
@@ -206,14 +281,16 @@ class DeepSpeech(nn.Module):
         window_size = self.audio_conf["window_size"]
         num_classes = len(self.labels)
 
-        self.conv = MaskConv(nn.Sequential(
-            nn.Conv2d(1, 32, kernel_size=(41, 11), stride=(2, 2), padding=(20, 5)),
-            nn.BatchNorm2d(32),
-            nn.Hardtanh(0, 20, inplace=True),
-            nn.Conv2d(32, 32, kernel_size=(21, 11), stride=(2, 1), padding=(10, 5)),
-            nn.BatchNorm2d(32),
-            nn.Hardtanh(0, 20, inplace=True)
-        ))
+        self.conv = MaskConv(
+            nn.Sequential(
+                nn.Conv2d(1, 32, kernel_size=(41, 11), stride=(2, 2), padding=(20, 5)),
+                nn.BatchNorm2d(32),
+                nn.Hardtanh(0, 20, inplace=True),
+                nn.Conv2d(32, 32, kernel_size=(21, 11), stride=(2, 1), padding=(10, 5)),
+                nn.BatchNorm2d(32),
+                nn.Hardtanh(0, 20, inplace=True),
+            )
+        )
         # Based on above convolutions and spectrogram size using conv formula (W - F + 2P)/ S+1
         rnn_input_size = int(math.floor((sample_rate * window_size) / 2) + 1)
         rnn_input_size = int(math.floor(rnn_input_size + 2 * 20 - 41) / 2 + 1)
@@ -221,23 +298,36 @@ class DeepSpeech(nn.Module):
         rnn_input_size *= 32
 
         rnns = []
-        rnn = BatchRNN(input_size=rnn_input_size, hidden_size=rnn_hidden_size, rnn_type=rnn_type,
-                       bidirectional=bidirectional, batch_norm=False)
-        rnns.append(('0', rnn))
+        rnn = BatchRNN(
+            input_size=rnn_input_size,
+            hidden_size=rnn_hidden_size,
+            rnn_type=rnn_type,
+            bidirectional=bidirectional,
+            batch_norm=False,
+        )
+        rnns.append(("0", rnn))
         for x in range(nb_layers - 1):
-            rnn = BatchRNN(input_size=rnn_hidden_size, hidden_size=rnn_hidden_size, rnn_type=rnn_type,
-                           bidirectional=bidirectional)
-            rnns.append(('%d' % (x + 1), rnn))
+            rnn = BatchRNN(
+                input_size=rnn_hidden_size,
+                hidden_size=rnn_hidden_size,
+                rnn_type=rnn_type,
+                bidirectional=bidirectional,
+            )
+            rnns.append(("%d" % (x + 1), rnn))
         self.rnns = nn.Sequential(OrderedDict(rnns))
-        self.lookahead = nn.Sequential(
-            # consider adding batch norm?
-            Lookahead(rnn_hidden_size, context=context),
-            nn.Hardtanh(0, 20, inplace=True)
-        ) if not bidirectional else None
+        self.lookahead = (
+            nn.Sequential(
+                # consider adding batch norm?
+                Lookahead(rnn_hidden_size, context=context),
+                nn.Hardtanh(0, 20, inplace=True),
+            )
+            if not bidirectional
+            else None
+        )
 
         fully_connected = nn.Sequential(
             nn.BatchNorm1d(rnn_hidden_size),
-            nn.Linear(rnn_hidden_size, num_classes, bias=False)
+            nn.Linear(rnn_hidden_size, num_classes, bias=False),
         )
         self.fc = nn.Sequential(
             SequenceWise(fully_connected),
@@ -250,7 +340,9 @@ class DeepSpeech(nn.Module):
         x, _ = self.conv(x, output_lengths)
 
         sizes = x.size()
-        x = x.view(sizes[0], sizes[1] * sizes[2], sizes[3])  # Collapse feature dimension
+        x = x.view(
+            sizes[0], sizes[1] * sizes[2], sizes[3]
+        )  # Collapse feature dimension
         x = x.transpose(1, 2).transpose(0, 1).contiguous()  # TxNxH
 
         for rnn in self.rnns:
@@ -275,10 +367,16 @@ class DeepSpeech(nn.Module):
         seq_len = input_length
         for m in self.conv.modules():
             if type(m) == nn.modules.conv.Conv2d:
-                seq_len = seq_len + 2 * m.padding[1] - m.dilation[1] * (m.kernel_size[1] - 1) - 1
+                seq_len = (
+                    seq_len
+                    + 2 * m.padding[1]
+                    - m.dilation[1] * (m.kernel_size[1] - 1)
+                    - 1
+                )
                 seq_len = seq_len.true_divide(m.stride[1]) + 1
         return seq_len.int()
 
+
 # Taken from https://github.com/pytorch/examples/blob/master/word_language_model/model.py#L108-L152
 class PositionalEncoding(nn.Module):
     r"""Inject some information about the relative or absolute position of the tokens
@@ -303,11 +401,13 @@ class PositionalEncoding(nn.Module):
 
         pe = torch.zeros(max_len, d_model)
         position = torch.arange(0, max_len, dtype=torch.float).unsqueeze(1)
-        div_term = torch.exp(torch.arange(0, d_model, 2).float() * (-math.log(10000.0) / d_model))
+        div_term = torch.exp(
+            torch.arange(0, d_model, 2).float() * (-math.log(10000.0) / d_model)
+        )
         pe[:, 0::2] = torch.sin(position * div_term)
         pe[:, 1::2] = torch.cos(position * div_term)
         pe = pe.unsqueeze(0).transpose(0, 1)
-        self.register_buffer('pe', pe)
+        self.register_buffer("pe", pe)
 
     def forward(self, x):
         r"""Inputs of forward function
@@ -320,9 +420,10 @@ class PositionalEncoding(nn.Module):
             >>> output = pos_encoder(x)
         """
 
-        x = x + self.pe[:x.size(0), :]
+        x = x + self.pe[: x.size(0), :]
         return self.dropout(x)
 
+
 class TransformerModel(nn.Module):
     """Container module with an encoder, a recurrent or transformer module, and a decoder."""
 
@@ -331,9 +432,10 @@ class TransformerModel(nn.Module):
         try:
             from torch.nn import TransformerEncoder, TransformerEncoderLayer
         except Exception as e:
-            raise ImportError('TransformerEncoder module does not exist in PyTorch 1.1 or '
-                              'lower.') from e
-        self.model_type = 'Transformer'
+            raise ImportError(
+                "TransformerEncoder module does not exist in PyTorch 1.1 or lower."
+            ) from e
+        self.model_type = "Transformer"
         self.src_mask = None
         self.pos_encoder = PositionalEncoding(ninp, dropout)
         encoder_layers = TransformerEncoderLayer(ninp, nhead, nhid, dropout)
@@ -356,7 +458,9 @@ class TransformerModel(nn.Module):
             device = src.device
             # This will be created once during warmup
             if self.src_mask is None or self.src_mask.size(0) != len(src):
-                mask = nn.Transformer.generate_square_subsequent_mask(len(src)).to(device)
+                mask = nn.Transformer.generate_square_subsequent_mask(len(src)).to(
+                    device
+                )
                 self.src_mask = mask
         else:
             self.src_mask = None
@@ -367,10 +471,11 @@ class TransformerModel(nn.Module):
         output = self.decoder(output)
         return F.log_softmax(output, dim=-1)
 
+
 # From https://github.com/pytorch/text/blob/master/torchtext/modules
 class MultiheadAttentionContainer(torch.nn.Module):
     def __init__(self, nhead, in_proj_container, attention_layer, out_proj):
-        r""" A multi-head attention container
+        r"""A multi-head attention container
         Args:
             nhead: the number of heads in the multiheadattention model
             in_proj_container: A container of multi-head in-projection linear layers (a.k.a nn.Linear).
@@ -398,10 +503,15 @@ class MultiheadAttentionContainer(torch.nn.Module):
         self.attention_layer = attention_layer
         self.out_proj = out_proj
 
-    def forward(self, query: torch.Tensor, key: torch.Tensor, value: torch.Tensor,
-                attn_mask: Optional[torch.Tensor] = None,
-                bias_k: Optional[torch.Tensor] = None,
-                bias_v: Optional[torch.Tensor] = None) -> Tuple[torch.Tensor, torch.Tensor]:
+    def forward(
+        self,
+        query: torch.Tensor,
+        key: torch.Tensor,
+        value: torch.Tensor,
+        attn_mask: Optional[torch.Tensor] = None,
+        bias_k: Optional[torch.Tensor] = None,
+        bias_v: Optional[torch.Tensor] = None,
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         r"""
         Args:
             query, key, value (Tensor): map a query and a set of key-value pairs to an output.
@@ -420,29 +530,40 @@ class MultiheadAttentionContainer(torch.nn.Module):
             where where L is the target length, S is the sequence length, H is the number of attention heads,
                 N is the batch size, and E is the embedding dimension.
         """
-        tgt_len, src_len, bsz, embed_dim = query.size(-3), key.size(-3), query.size(-2), query.size(-1)
+        tgt_len, src_len, bsz, embed_dim = (
+            query.size(-3),
+            key.size(-3),
+            query.size(-2),
+            query.size(-1),
+        )
         q, k, v = self.in_proj_container(query, key, value)
-        assert q.size(-1) % self.nhead == 0, "query's embed_dim must be divisible by the number of heads"
+        assert (
+            q.size(-1) % self.nhead == 0
+        ), "query's embed_dim must be divisible by the number of heads"
         head_dim = q.size(-1) // self.nhead
         q = q.reshape(tgt_len, bsz * self.nhead, head_dim)
 
-        assert k.size(-1) % self.nhead == 0, "key's embed_dim must be divisible by the number of heads"
+        assert (
+            k.size(-1) % self.nhead == 0
+        ), "key's embed_dim must be divisible by the number of heads"
         head_dim = k.size(-1) // self.nhead
         k = k.reshape(src_len, bsz * self.nhead, head_dim)
 
-        assert v.size(-1) % self.nhead == 0, "value's embed_dim must be divisible by the number of heads"
+        assert (
+            v.size(-1) % self.nhead == 0
+        ), "value's embed_dim must be divisible by the number of heads"
         head_dim = v.size(-1) // self.nhead
         v = v.reshape(src_len, bsz * self.nhead, head_dim)
 
-        attn_output, attn_output_weights = self.attention_layer(q, k, v, attn_mask=attn_mask,
-                                                                bias_k=bias_k, bias_v=bias_v)
+        attn_output, attn_output_weights = self.attention_layer(
+            q, k, v, attn_mask=attn_mask, bias_k=bias_k, bias_v=bias_v
+        )
         attn_output = attn_output.reshape(tgt_len, bsz, embed_dim)
         attn_output = self.out_proj(attn_output)
         return attn_output, attn_output_weights
 
 
 class ScaledDotProduct(torch.nn.Module):
-
     def __init__(self, dropout=0.0):
         r"""Processes a projected query and key-value pair to apply
         scaled dot product attention.
@@ -459,10 +580,15 @@ class ScaledDotProduct(torch.nn.Module):
         super().__init__()
         self.dropout = dropout
 
-    def forward(self, query: torch.Tensor, key: torch.Tensor, value: torch.Tensor,
-                attn_mask: Optional[torch.Tensor] = None,
-                bias_k: Optional[torch.Tensor] = None,
-                bias_v: Optional[torch.Tensor] = None) -> Tuple[torch.Tensor, torch.Tensor]:
+    def forward(
+        self,
+        query: torch.Tensor,
+        key: torch.Tensor,
+        value: torch.Tensor,
+        attn_mask: Optional[torch.Tensor] = None,
+        bias_k: Optional[torch.Tensor] = None,
+        bias_v: Optional[torch.Tensor] = None,
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         r"""Uses a scaled dot product with the projected key-value pair to update
         the projected query.
         Args:
@@ -485,10 +611,16 @@ class ScaledDotProduct(torch.nn.Module):
             of attention heads, N is the batch size, and E is the embedding dimension.
         """
         if bias_k is not None and bias_v is not None:
-            assert key.size(-1) == bias_k.size(-1) and key.size(-2) == bias_k.size(-2) and bias_k.size(-3) == 1, \
-                "Shape of bias_k is not supported"
-            assert value.size(-1) == bias_v.size(-1) and value.size(-2) == bias_v.size(-2) and bias_v.size(-3) == 1, \
-                "Shape of bias_v is not supported"
+            assert (
+                key.size(-1) == bias_k.size(-1)
+                and key.size(-2) == bias_k.size(-2)
+                and bias_k.size(-3) == 1
+            ), "Shape of bias_k is not supported"
+            assert (
+                value.size(-1) == bias_v.size(-1)
+                and value.size(-2) == bias_v.size(-2)
+                and bias_v.size(-3) == 1
+            ), "Shape of bias_v is not supported"
             key = torch.cat([key, bias_k])
             value = torch.cat([value, bias_v])
             if attn_mask is not None:
@@ -496,29 +628,43 @@ class ScaledDotProduct(torch.nn.Module):
                 attn_mask = torch.nn.functional.pad(_attn_mask, [0, 1])
 
         tgt_len, head_dim = query.size(-3), query.size(-1)
-        assert query.size(-1) == key.size(-1) == value.size(-1), "The feature dim of query, key, value must be equal."
+        assert (
+            query.size(-1) == key.size(-1) == value.size(-1)
+        ), "The feature dim of query, key, value must be equal."
         assert key.size() == value.size(), "Shape of key, value must match"
         src_len = key.size(-3)
         batch_heads = max(query.size(-2), key.size(-2))
 
         # Scale query
-        query, key, value = query.transpose(-2, -3), key.transpose(-2, -3), value.transpose(-2, -3)
+        query, key, value = (
+            query.transpose(-2, -3),
+            key.transpose(-2, -3),
+            value.transpose(-2, -3),
+        )
         query = query * (float(head_dim) ** -0.5)
         if attn_mask is not None:
             if attn_mask.dim() != 3:
-                raise RuntimeError('attn_mask must be a 3D tensor.')
-            if (attn_mask.size(-1) != src_len) or (attn_mask.size(-2) != tgt_len) or \
-               (attn_mask.size(-3) != 1 and attn_mask.size(-3) != batch_heads):
-                raise RuntimeError('The size of the attn_mask is not correct.')
+                raise RuntimeError("attn_mask must be a 3D tensor.")
+            if (
+                (attn_mask.size(-1) != src_len)
+                or (attn_mask.size(-2) != tgt_len)
+                or (attn_mask.size(-3) != 1 and attn_mask.size(-3) != batch_heads)
+            ):
+                raise RuntimeError("The size of the attn_mask is not correct.")
             if attn_mask.dtype != torch.bool:
-                raise RuntimeError('Only bool tensor is supported for attn_mask')
+                raise RuntimeError("Only bool tensor is supported for attn_mask")
 
         # Dot product of q, k
         attn_output_weights = torch.matmul(query, key.mT)
         if attn_mask is not None:
-            attn_output_weights.masked_fill_(attn_mask, -1e8,)
+            attn_output_weights.masked_fill_(
+                attn_mask,
+                -1e8,
+            )
         attn_output_weights = torch.nn.functional.softmax(attn_output_weights, dim=-1)
-        attn_output_weights = torch.nn.functional.dropout(attn_output_weights, p=self.dropout, training=self.training)
+        attn_output_weights = torch.nn.functional.dropout(
+            attn_output_weights, p=self.dropout, training=self.training
+        )
         attn_output = torch.matmul(attn_output_weights, value)
         return attn_output.transpose(-2, -3), attn_output_weights
 
@@ -537,10 +683,9 @@ class InProjContainer(torch.nn.Module):
         self.key_proj = key_proj
         self.value_proj = value_proj
 
-    def forward(self,
-                query: torch.Tensor,
-                key: torch.Tensor,
-                value: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
+    def forward(
+        self, query: torch.Tensor, key: torch.Tensor, value: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
         r"""Projects the input sequences using in-proj layers.
         Args:
             query, key, value (Tensors): sequence to be projected
